@@ -4,13 +4,15 @@ use Mojo::Base -base;
 
 use Carp ();
 use Mojo::Collection;
+use Mojo::Promise;
 use Mojo::Redfish::Client::Result;
 use Scalar::Util ();
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 $VERSION = eval $VERSION;
 
-has ssl  => 1;
+has concurrency => 5;
+has ssl => 1;
 has [qw/host password token username/];
 
 has ua => sub {
@@ -48,11 +50,30 @@ sub get {
   return $self->_result($tx->res->json);
 }
 
+sub get_p {
+  my ($self, $url) = @_;
+  $self->ua->get_p($url)->then(sub{
+    my $tx = shift;
+    if (my $err = $tx->error) { Carp::croak $err->{message} }
+    my $data = $tx->res->json;
+    return $self->_result($tx->res->json);
+  });
+}
+
 sub root {
   my $self = shift;
   return $self->{root} ||= $self->get('/redfish/v1');
 }
 
+sub root_p {
+  my $self = shift;
+  return Mojo::Promise->resolve($self->{root})
+    if $self->{root};
+  return $self->get_p('/redfish/v1')->then(sub{
+    return $self->{root} = shift;
+  });
+}
+
 sub _result {
   my ($self, $data) = @_;
   return Mojo::Redfish::Client::Result->new(
@@ -95,6 +116,14 @@ This is still a work-in-progress, however the author uses it in work application
 
 L<Mojo::Redfish::Client> inherits all attributes from L<Mojo::Base> and implements the following new ones.
 
+=head2 concurrency
+
+The B<per-operation> concurrency limit.
+This is not a global concurrency, it only limits concurrency when a single operation would make several concurrent requests within it.
+For example (and the only current case), L<Mojo::Redfish::Client/get_p> on an array.
+If zero (or otherwise falsey), no concurrency limit will be applied.
+Default is C<5>.
+
 =head2 host
 
 The Redfish host.
@@ -135,6 +164,10 @@ Requests the requested url via the L</ua>.
 Returns an instance of L<Mojo::Redfish::Client::Result>.
 Dies on errors (the exact exception and behavior is subject to change).
 
+=head2 get_p
+
+Same as L</get> but returns a L<Mojo::Promise> that resolves to the result.
+
 =head2 root
 
   my $result = $client->root;
@@ -145,6 +178,10 @@ Caches and returns the result.
   # same as (except for the caching)
   my $result = $client->get('/redfish/v1');
 
+=head2 root_p
+
+Same as L</root> but returns a L<Mojo::Promise> that resolves to the (possibly cached) root result.
+
 =head1 FUTURE WORK
 
 This module is still in early development.
@@ -154,16 +191,8 @@ Future work will include
 
 =item *
 
-Non-blocking (promise-based) api
-
-=item *
-
 Session management
 
-=item *
-
-Even more testing
-
 =back
 
 
